using UnityEngine;
using System.Collections;

public class AIBehavior {
	protected Actor actor;
	
	public void SetActor(Actor a)
	{
		actor = a;
	}
	
	public virtual void GetSteering(ref SteeringOutput output)
	{
		
	}
}

// Here are a few calls you are going to need
// To get the target position  -> actor.GetTarget().position;
// To get our current position -> actor.GetGameObject().transform.position;

public class KinematicSeek : AIBehavior
{
	public override void GetSteering(ref SteeringOutput output)
	{
		// First work out the direction
        //output.linear = /* ? */ ;

        // If there is no direction, do nothing
        if (output.linear.sqrMagnitude > 0)
        {
            output.linear.Normalize();
            output.linear *= actor.GetMaxSpeed();
        }
	}
}

public class KinematicFlee : AIBehavior
{
	public override void GetSteering(ref SteeringOutput output)
	{
		// First work out the direction
        //output.linear = /* ? */;

        // If there is no direction, do nothing
        if (output.linear.sqrMagnitude > 0)
        {
            output.linear.Normalize();
            output.linear *= actor.GetMaxSpeed();
        }
	}
}

public class KinematicArrive : AIBehavior
{
	public float radius = 1.0f;
	public float timeToTarget = 1.0f;
	
	public override void GetSteering(ref SteeringOutput output)
	{
		// First work out the direction
        //output.linear = /* ? */;

        // If there is no direction, do nothing
        if (output.linear.sqrMagnitude < radius*radius)
        {
            output.linear = Vector3.zero;
        }
        else
        {
        	float maxSpeed = actor.GetMaxSpeed();
            // We'd like to arrive in timeToTarget seconds
            //output.linear *= /* ? */;

            // If that is too fast, then clip the speed
            if (output.linear.sqrMagnitude > maxSpeed*maxSpeed)
            {
                output.linear.Normalize();
                output.linear *= maxSpeed;
            }
        }
	}
}

//For this behavior make sure that you set FaceWhereYouAreGoing = false in Actor IN THE EDITOR.
public class KinematicWander : AIBehavior
{
	public override void GetSteering(ref SteeringOutput output)
	{
		// Move forward in the current direction
		float currentActorOrientation = actor.GetOrientation();
		Vector3 fwd = Actor.GetOrientationAsVector(currentActorOrientation);
		
        //output.linear = /* ? */;

        // Turn a little
        float change = Random.value - Random.value;
        //output.angular = /* ? */;
	}
	
	
	
}

public class KinematicAlign : AIBehavior
{
	public float timeToTarget = 0.1f;
	public override void GetSteering(ref SteeringOutput output)
	{
		Actor other = null;
		Transform t = actor.GetTarget();
		other = (Actor)t.GetComponent("Actor");
		if(other)
		{
	        //float change = /* ? */;
	        //output.angular = /* ? */;
		}
	}
}