using UnityEngine;
using System.Collections;

public class Actor : MonoBehaviour{
	public Transform target;
	public float maxSpeed;
	public float maxRotation;
	public string desiredBehavior = "Seek";
	public bool faceWhereYouAreGoing = true;
	public Transform modelToRotate;
	public GameObject world;
	public Color color;
	
	private SteeringOutput steering = new SteeringOutput();
	private AIBehavior behavior;
	private float orientation;
	private Vector3 velocity;
	private Vector3 lastPosition;
	private Matrix4x4 xform;
	
	void Awake()
	{
		if(desiredBehavior.ToUpper() == "SEEK")
		{
			behavior = new KinematicSeek();
		}
		else if(desiredBehavior.ToUpper() == "FLEE")
		{
			behavior = new KinematicFlee();
		}
		else if(desiredBehavior.ToUpper() == "ARRIVE")
		{
			behavior = new KinematicArrive();
		}
		else if(desiredBehavior.ToUpper() == "WANDER")
		{
			behavior = new KinematicWander();
		}
		else if(desiredBehavior.ToUpper() == "ALIGN")
		{
			behavior = new KinematicAlign();
		}
		modelToRotate.gameObject.renderer.material.color = color;
		behavior.SetActor(this);
		velocity = Vector3.zero;
		;
	}
	
	void Update()
	{
		Tick();
	}
	
	public GameObject GetGameObject()
	{
		return gameObject;
	}
	
	public Transform GetTarget()
	{
		return target;
	}
	
	public float GetMaxSpeed()
	{
		return maxSpeed;
	}
	
	public float GetMaxRotation()
	{
		return maxRotation;
	}
	
	public float GetOrientation()
	{
		return orientation;
	}
	
	public void Tick()
	{
		steering.Reset();
		behavior.GetSteering(ref steering);
		velocity = steering.linear;
		orientation += steering.angular;
		
		transform.Translate(velocity*Time.deltaTime, Space.World);
		transform.rotation = Quaternion.identity;
		transform.Rotate(Vector3.up*orientation*Mathf.Rad2Deg);
		
		WrapPositionToWorld();
		
		if(faceWhereYouAreGoing)
			FaceWhereYouAreGoing();
	}
	
	private void WrapPositionToWorld()
	{
		Vector3 p = transform.position;
		
		float x = p.x;
		if(x < world.collider.bounds.min.x)
			x = world.collider.bounds.max.x;
		else if(x > world.collider.bounds.max.x)
			x = world.collider.bounds.min.x;
		
		float z = p.z;
		if(z < world.collider.bounds.min.z)
			z = world.collider.bounds.min.z;
		else if(z > world.collider.bounds.max.z)
			z = world.collider.bounds.min.z;
			
		transform.position = new Vector3(x, 4.0f,z);	
	}
	
	private void FaceWhereYouAreGoing()
	{
		if(velocity.sqrMagnitude > 0)
		{
			Vector3 movement = velocity;
			
			float facingAngle = Mathf.Atan2(movement.x,movement.z)* Mathf.Rad2Deg;
			
			//Reset rotation
			modelToRotate.transform.rotation = Quaternion.identity;
			
			//Rotate to new heading
			modelToRotate.transform.Rotate(0,facingAngle,0, Space.World);
			orientation = facingAngle;
			
		}
		
	}
	
	public static Vector3 GetOrientationAsVector(float orientation)
	{
        return new Vector3(Mathf.Sin(orientation),
                       0,
                       Mathf.Cos(orientation));	
	}
}
